package costo.kml2java.monitor.gui.basecomponents;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Paint;
import java.awt.Stroke;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;

import org.apache.commons.collections15.Transformer;

import costo.kml2java.framework.ExecutableLTS;
import costo.kml2java.framework.IProvidedService;
import costo.kml2java.framework.IService;
import costo.kml2java.framework.IServiceEvolutionListener;
import costo.kml2java.framework.StepByStepPauseManager;
import costo.kml2java.framework.channels.IChannel;
import costo.kml2java.framework.exceptions.ServiceException;
import edu.uci.ics.jung.algorithms.layout.ISOMLayout;
import edu.uci.ics.jung.algorithms.layout.Layout;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.SparseMultigraph;
import edu.uci.ics.jung.graph.util.EdgeType;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.DefaultModalGraphMouse;
import edu.uci.ics.jung.visualization.control.ModalGraphMouse.Mode;
import edu.uci.ics.jung.visualization.decorators.ToStringLabeller;
import edu.uci.ics.jung.visualization.renderers.Renderer.VertexLabel.Position;

public class JExecutableServicePanel extends JPanel implements IServiceEvolutionListener, ActionListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private final Map<String, MyNode> NODES = new HashMap<String, MyNode>();
	private final Map<String, MyLink> LINKS = new HashMap<String, MyLink>();
	private Graph<MyNode, MyLink> graph;
	private Layout<MyNode, MyLink> layout;
	private VisualizationViewer<MyNode, MyLink> visuserver;

	private JTextArea servicestate;
	private JTextArea ta;

	private IProvidedService serv;
	private JButton start, reset;
	private StatePanel statepanel;
    private StepByStepPauseManager pausemanager = new StepByStepPauseManager();
	
    public JExecutableServicePanel(IProvidedService serv) {
		this.serv = serv;
		this.serv.addServiceListener(this);
		this.serv.setPauseManager(pausemanager);
		graph = new SparseMultigraph<MyNode, MyLink>();
		createGraphFromLTS(serv.getLTS());
		init();
		// mytests();
	}

	public void createGraphFromLTS(ExecutableLTS lts) {
		if (lts == null) {
			// System.err.println("no LTS for"+this.serv);

			createVoidGraph();
			return;
		}
		// Graph<MyNode, MyLink> graph = new SparseMultigraph<MyNode, MyLink>();
		for (String s : lts.getOutGoing().keySet()) {
			MyNode n = new MyNode(s);
			graph.addVertex(n);
			NODES.put(s, n);

		}
		for (Map.Entry<String, String> tlab : lts.getTransitionLabels().entrySet()) {
			String[] tab = tlab.getKey().split("___");
			MyLink ln = new MyLink(tlab.getKey());
			ln.setLabel(tlab.getValue());
			graph.addEdge(ln, NODES.get(tab[0]), NODES.get(tab[1]), EdgeType.DIRECTED);
			LINKS.put(tlab.getKey(), ln);
		}

	}

	private void createVoidGraph() {
		// TODO Auto-generated method stub

		MyNode n = new MyNode("no");
		graph.addVertex(n);
		NODES.put("no", n);

		MyLink ln = new MyLink("no_no");
		ln.setLabel("nop");
		graph.addEdge(ln, n, n, EdgeType.DIRECTED);
		LINKS.put("no_no", ln);

	}

	public void init() {

		// start = new JButton("start");
		// start.addActionListener(this);
		// reset = new JButton("reset");
		// reset.addActionListener(this);
		// // graph = createGraphFromFile(ltsfile);
		layout = new ISOMLayout<MyNode, MyLink>(graph);
		layout.setSize(new Dimension(300, 300));
		// layout.
		visuserver = new VisualizationViewer<MyNode, MyLink>(layout);
		visuserver.setPreferredSize(new Dimension(350, 350));
		Transformer<MyNode, Paint> nodecolor = new Transformer<MyNode, Paint>() {

			@Override
			public Paint transform(MyNode arg0) {
				if (arg0.isSelected())
					return Color.RED;
				return Color.GREEN;

			}
		};

		Transformer<MyLink, Paint> edgecolor = new Transformer<MyLink, Paint>() {

			@Override
			public Paint transform(MyLink arg0) {
				if (arg0.isSelected())
					return Color.RED;
				return Color.GREEN;

			}
		};

		float dash[] = { 10.0f };
		final Stroke edgeStroke = new BasicStroke(1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10.0f, dash, 0.0f);
		Transformer<MyLink, Stroke> edgeStrokeTransformer = new Transformer<MyLink, Stroke>() {
			public Stroke transform(MyLink s) {

				return edgeStroke;

			}
		};
		Transformer<MyLink, String> edgeLabelTransformer = new Transformer<MyLink, String>() {

			@Override
			public String transform(MyLink arg0) {

				return (arg0.isSelected() ? arg0.getLabel() : arg0.getName());
			}
		};
		visuserver.getRenderContext().setEdgeDrawPaintTransformer(edgecolor);
		// System.out.println();
		visuserver.getRenderContext().setVertexDrawPaintTransformer(nodecolor);
		visuserver.getRenderContext().setVertexFillPaintTransformer(nodecolor);
		visuserver.getRenderContext().setVertexLabelTransformer(new ToStringLabeller());
		visuserver.getRenderContext().setEdgeLabelTransformer(edgeLabelTransformer);
		visuserver.getRenderer().getVertexLabelRenderer().setPosition(Position.CNTR);
		DefaultModalGraphMouse<MyNode, MyLink> gm = new DefaultModalGraphMouse();
		gm.setMode(Mode.TRANSFORMING);
		// gm.setMode(ModalGraphMouse.Mode.TRANSFORMING);
		visuserver.setGraphMouse(gm);
		visuserver.addKeyListener(gm.getModeKeyListener());

		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.add(getToolbar());
		// this.setLayout(new BorderLayout());
		ta = new JTextArea("test", 15, 100);
		JScrollPane p = new JScrollPane(ta);

		p.setPreferredSize(new Dimension(500, 100));

		// this.add(p, BorderLayout.SOUTH);
		this.statepanel = new StatePanel(serv);
		JScrollPane psp = new JScrollPane(statepanel);
		psp.setPreferredSize(new Dimension(500, 100));
		servicestate = new JTextArea(serv.toString() + "   \n \n                     ");

		// this.add(servicestate, BorderLayout.NORTH);
		this.setPreferredSize(new Dimension(800, 800));

		// this.add(visuserver, BorderLayout.CENTER);
		this.add(visuserver);
		this.add(psp);
		this.add(p);

	}

	private Component getToolbar() {
		JToolBar jbar = new JToolBar("Debug");
		DemoAction stepbystep = new DemoAction("Next step", null, "reach next state if possible", 'N');
		jbar.add(stepbystep);
		return jbar;
	}

	public IProvidedService getServ() {
		return serv;
	}

	@Override
	public void transitionFired(String tname, String currentState, IService serv, String log, Exception ex) {

		ta.append("\n" + log);
		if (ex != null) {
			ta.append("Erreur: " + ex.getMessage());
		}
		ta.revalidate();
		servicestate.setText(serv.toString());
		visuserver.getModel().getRelaxer().pause();
		for (MyNode node : NODES.values()) {
			node.setSelected(false);

		}

		NODES.get(currentState).setSelected(true);
		for (MyLink ln : LINKS.values()) {
			ln.setSelected(false);

		}
		MyLink selectedln = LINKS.get(tname);
		// if (selectedln==null) System.err.println("transition "+ tname+" not found");else
		if (selectedln != null) {
			selectedln.setSelected(true);
		}
		visuserver.getModel().getRelaxer().resume();
		visuserver.repaint();
		if (((IProvidedService) serv).getLTS().finalStateReached()) {
			if (!serv.isPostConditionSatisfied()) {
				new JDialog((JFrame) null, serv.getName() + " post condition  violated");
			}
		}
		// frame.validate();
		// Probleme avec le dolayout
		updateFields();
	}

	private void updateFields() {
		this.statepanel.stateChanged(null);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == start) {
			try {
				serv.start();
			} catch (ServiceException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}

		if (e.getSource() == reset) {
			ta.setText("");
			serv.reset();
		}

		if (e.getActionCommand().equals("blocking_calls")) {
			serv.setBlockingCalls(true);

		}

		if (e.getActionCommand().equals("non_blocking_calls")) {
			serv.setBlockingCalls(false);
		}
	}

	class DemoAction extends AbstractAction {

	    /**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public DemoAction(String text, Icon icon, String description,
	        char accelerator) {
	      super(text, icon);
	      putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(accelerator,
	          Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
	      putValue(SHORT_DESCRIPTION, description);
	    }

	    public void actionPerformed(ActionEvent e) {
	    	System.out.println(e.getActionCommand());
	    	if (e.getActionCommand().equals("Next step")){
	    		pausemanager.setPauseMode(true);
	    		serv.setPaused(false);
	    		
	    	}
	     
	    }
	  }

	@Override
	public void serviceError(IService serv, IChannel source, String message) {
			System.err.println("--SERV-error>> "+message);
	}
}
