package costo.kml2java.monitor.gui.multipleframes;

import javax.swing.JDesktopPane;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JMenuBar;
import javax.swing.JFrame;
import javax.swing.KeyStroke;

import costo.kml2java.framework.ExecutableComponent;
import costo.kml2java.framework.IProvidedService;
import costo.kml2java.framework.IService;
import costo.kml2java.framework.IServiceEvolutionListener;
import costo.kml2java.framework.channels.IChannel;
import costo.kml2java.monitor.gui.basecomponents.JExecutableServicePanel;
import costo.kml2java.monitor.gui.basecomponents.ServiceMonitoringInfo;

import java.awt.event.*;
import java.awt.*;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Properties;
import java.util.Map;


/*
 * InternalFrameDemo.java requires:
 *   MyInternalFrame.java
 */
public class ComponentFrame extends JFrame
                               implements ActionListener, IServiceEvolutionListener {
    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	JDesktopPane desktop;
	private ExecutableComponent component;
	private JLabel compstate;
	private int openframecount;
	private Properties servicesprop;
	private Map<IProvidedService,ServiceMonitoringInfo> servinfos= new HashMap<IProvidedService, ServiceMonitoringInfo>();

    public ComponentFrame() {
        super("InternalFrameDemo");

        //Make the big window be indented 50 pixels from each edge
        //of the screen.
        int inset = 50;
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds(inset, inset,
                  screenSize.width  - inset*2,
                  screenSize.height - inset*2);

        //Set up the GUI.
        desktop = new JDesktopPane(); //a specialized layered pane
    //    createFrame(); //create first "window"
        getContentPane().setLayout(new BorderLayout());
        add(desktop, BorderLayout.CENTER);
        compstate= new JLabel("no component loaded yet");
        add(compstate,BorderLayout.SOUTH);
        setJMenuBar(createMenuBar());

        //Make dragging a little faster but perhaps uglier.
        desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
    }

    public ComponentFrame(String name) {
		
	}

	protected JMenuBar createMenuBar() {
        JMenuBar menuBar = new JMenuBar();

        //Set up the lone menu.
        JMenu menu = new JMenu("Component");
        menu.setMnemonic(KeyEvent.VK_D);
        menuBar.add(menu);

        //Set up the first menu item.
        JMenuItem menuItem = new JMenuItem("Reset");
        menuItem.setMnemonic(KeyEvent.VK_R);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_R, ActionEvent.ALT_MASK));
        menuItem.setActionCommand("reset");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Pause ");
        menuItem.setMnemonic(KeyEvent.VK_P);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_P, ActionEvent.ALT_MASK));
        menuItem.setActionCommand("pause");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Un_Pause ");
        menuItem.setMnemonic(KeyEvent.VK_U);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_U, ActionEvent.ALT_MASK));
        menuItem.setActionCommand("unpause");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        
        menuItem = new JMenuItem("Stop ");
        menuItem.setMnemonic(KeyEvent.VK_S);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_S, ActionEvent.ALT_MASK));
        menuItem.setActionCommand("stop");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        //Set up the second menu item.
        menuItem = new JMenuItem("quit");
        menuItem.setMnemonic(KeyEvent.VK_Q);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_Q, ActionEvent.ALT_MASK));
        menuItem.setActionCommand("quit");
        menuItem.addActionListener(this);
        menu.add(menuItem);

        return menuBar;
    }

    //React to menu selections.
    public void actionPerformed(ActionEvent e) {
    	
        if ("stop".equals(e.getActionCommand())) { //new
        	this.component.stop();
        	return;
        }

        if ("reset".equals(e.getActionCommand())) { //new
        	this.component.reset();
        	return;
        }
        if ("pause".equals(e.getActionCommand())) { //new
        	this.component.setPaused(true);
        	return;
        }
        if ("unpause".equals(e.getActionCommand())) { //new
        	this.component.setPaused(false);
        	return;
        }
        if ("quit".equals(e.getActionCommand())) { //new
        	this.quit();
        	return;
        }
    }

    //Create a new internal frame.
    public  void createFrame(IProvidedService serv) {
    	
        ServiceInternalFrame frame = new ServiceInternalFrame(new JExecutableServicePanel(serv), openframecount);
        openframecount++;
        frame.setVisible(true); //necessary as of 1.3
        desktop.add(frame);
        try {
            frame.setSelected(true);
        } catch (java.beans.PropertyVetoException e) {
        	e.printStackTrace();
        }
    }

    //Quit the application.
    protected void quit() {
        System.exit(0);
    }
/**
 *  comp should have been initialized without recreating services
 * @param comp
 */
	public void assignComponent(ExecutableComponent comp) {
	this.component = comp;
//	this.component.init();
	this.compstate.setText(comp.toString());
	this.setTitle(comp.getName());
	getMonitoringValues(comp.getName());
	for (IProvidedService serv : this.component.getProvidedServices()){
	//	serv.init();
		String servinfo=servicesprop.getProperty(serv.getName());
		if (servinfo!=null) servinfos.put(serv, new ServiceMonitoringInfo(serv, servinfo));
		manageservice(serv);
		//serv.addServiceListener(this);
	//	this.createFrame(serv);
	}
		this.pack();
	}



	private void manageservice(IProvidedService serv) {
	  ServiceMonitoringInfo info= this.servinfos.get(serv);
	  if (info==null) return;
	  if (info.isMonitored()) serv.addServiceListener(this);
	  if (info.isWindow())  createFrame(serv);
	  if (info.isPaused()) serv.setPaused(true);
	
}

	private void getMonitoringValues(String name) {
		servicesprop = new Properties();
		try {
			servicesprop.load(new FileReader(name+".cmconf"));
			
		} catch (FileNotFoundException e) {
			System.err.println("no config file found for "+name+".cmconf");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	
}

	@Override
	public void transitionFired(String tname, String currentState,
			IService serv, String log, Exception ex) {
		this.compstate.setText(component.toString());
		System.out.println(log);
		if (ex!=null) System.err.println(log+ex.getMessage());
	}

	@Override
	public void serviceError(IService serv, IChannel source, String message) {
		// TODO Auto-generated method stub
		//modified by PA
		System.err.println("--SERV-error>> "+message);
		
	}

   
}
